/* Part 2 - separated into 2 parts to overcome 32K segment limitation
            on Macintosh */
/************************************************************************
*      H2O property functions based on:
*          L. Harr, J. Gallagher and G. Kell, "NBS/NRC Steam Tables,
*          Thermodynamic and Transport Properties and Computer Programs
*          for Vapor and Liquid States of Water in SI Units",
*          Hemisphere Publishing Corp., McGraw-Hill International Book
*          Company, 1984, ISBN 0-89116-354-9.
*
*      These functions are low-order fits that are fast running and
*      reasonably accurate (usually < 0.1% error). For more information
*      refer to the following publications:
*
*
*      Please feel free to distribute copies of these functions since the
*      general idea is to promote the free exchange of useful code.
*      Please DO NOT distribute altered copies; the world is confusing
*      enough as it is. I would appreciate being informed of any errors,
*      ommisions, comments, etc.
*
*          Wm. J. Garland
*          Associate Professor
*          Department of Engineering Physics
*          McMaster University
*          1280 Main St. W.
*          Hamilton, Ontario
*          Canada    L8S 4M1
*          Ph: (416) 525-9140 ext. 4925
*
*************************************************************************
*
* Saturation properties based on reference 1 (Garland and Hoskins)
*
* Subcooled and superheated properties based on reference 2 (Garland and
* Hand)
*         SAT means saturated
*         SUB means subcooled
*         SUP means superheated
*
***********************************************************************/
#include <math.h>
#include "h2o.h"
#define  ABS(x) ((x) >= 0 ? (x) : -(x))
/**********************************************************************/
float DVGDP(float P)
/* This function calculates the derivative with respect to
   pressure of the specific volume of light water in the gas
   phase at saturation for a given pressure.*/
{
     extern char *ERRH2O;
     float DVGDP,DDGDP;
     float VG(float P);
     if (P < 0.002)     {
       DDGDP= 5.0981616 * 0.936226 * pow(0.002,0.936226-1.0);
       DVGDP=-1.0* VG(0.002)*VG(0.002) *DDGDP;
       ERRH2O=" DVGDP: PRESSURE LESS THAN 0.002 MPa ";
     }
     else if (P > 21.5)     {
       DDGDP= 3.558734E-16* 13.03774* pow(21.5,12.03774);
       DVGDP=-1.0* VG(21.5)*VG(21.5) *DDGDP;
       ERRH2O=" DVGDP: PRESSURE GREATER THAN 21.5 MPa ";
     }
     else if (P >= 0.002  &&  P < 0.2139)     {
       DDGDP= 5.0981616 * 0.936226 * pow(P,0.936226-1.0);
       DVGDP=-1.0 * VG(P)*VG(P) *DDGDP;
     }
     else if (P >= 0.2139 && P < 1.112)     {
       DDGDP= 5.126076*0.9475862* pow(P,0.9475862 -1.0);
       DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else if (P >= 1.112 && P < 3.932)     {
       DDGDP= 4.630832*1.038819* pow(P,0.038819);
       DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else if (P >= 3.932 && P < 8.996)     {
       DDGDP= 2.868721*1.252148* pow(P,0.252148);
       DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else if (P >= 8.996 && P < 14.628)     {
       DDGDP= 0.5497653*1.831182* pow(P,0.831182);
       DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else if (P >= 14.628 && P <= 18.21)     {
       DDGDP= 8.5791582E-3* 3.176484* pow(P,2.176484);
       DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else if (P > 18.21 && P <= 20.253)     {
       DDGDP= 3.5587113E-6* 5.660939* pow(P,4.660939);
       DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     else    {
       DDGDP= 3.558734E-16* 13.03774* pow(P,12.03774);
       DVGDP=-1.0* VG(P)*VG(P) *DDGDP;
     }
     return(DVGDP);
}   /* end of function DVGDP(P) */

/**********************************************************************/
float HF(float P)
/* This function calculates the specific enthalpy of light water
   in the liquid phase at saturation for a given pressure.*/
{
     extern char *ERRH2O;
     float HF;
     if (P < 0.002)     {
       HF= 1128.777 * pow(0.002,0.135196) - 413.72;
       ERRH2O=" HF: PRESSURE LESS THAN 0.002 MPa ";
     }
     else if (P > 21.7)     {
       HF= 9.1417257E-14* pow(21.7,11.47287) +1752.0;
       ERRH2O=" HF: PRESSURE GREATER THAN 21.7 MPa ";
     }
     else if (P >= 0.002  &&  P < 0.0173)     {
       HF= 1128.777 * pow(P,0.135196) - 413.72;
     }
     else if (P >= 0.0173  &&  P < 0.1028)     {
       HF= 1050.7085 * pow(P,0.161797) - 306.5;
     }
     else if (P >= 0.1028 && P < 0.942)     {
       HF= 912.1779* pow(P,0.2061637) -150.0;
     }
     else if (P >= 0.942 && P < 4.02)     {
       HF= 638.0621* pow(P,0.2963192) +125.0;
     }
     else if (P >= 4.02 && P < 9.964)     {
       HF= 373.7665* pow(P,0.4235532) +415.0;
     }
     else if (P >= 9.964 && P < 16.673)     {
       HF= 75.38673* pow(P,0.8282384) +900.0;
     }
     else if (P >= 16.673 && P < 20.396)     {
       HF= 0.1150827* pow(P,2.711412) +1440.0;
     }
     else     {
       HF= 9.1417257E-14* pow(P,11.47287) +1752.0;
     }
     return(HF);
}   /* end of function HF(P) */

/**********************************************************************/
float HFSUB(float P,float T)
/* This function calculates the enthalpy of light water
   in the subcooled liquid phase.*/
{
      extern char *ERRH2O;
      float PP,TT,PS,HFSUB;
      float PSAT(float T);
      float HF(float P);
      PP=P;
      TT=T;
      PS=PSAT(T);
      if (P < PS)     {
        PP=PS;
        ERRH2O=" HFSUB: PRESSURE LESS THAN SATURATION ";
      }
  else if (P > 18.)     {
        PP=18.;
        ERRH2O=" HFSUB: PRESSURE GREATER THAN 18 MPa ";
      }
      else if (T < 17.511)     {
        TT=17.511;
        ERRH2O=" HFSUB: TEMPERATURE LESS THAN 17.511 deg C ";
      }
      else if (T > 357.03)     {
        TT=357.03;
        ERRH2O=" HFSUB: TEMPERATURE GREATER THAN 357.03 deg C ";
      }
      HFSUB = HF(PS) + (1.4-169./(369.-TT))*(PP-PS);
      return(HFSUB);
}   /* end of function HFSUB(P,T) */

/**********************************************************************/
float HG(float P)
/* This function calculates the specific enthalpy of light water
   in the gas phase at saturation for a given pressure.*/
{
      extern char *ERRH2O;
      float HG;
      if (P < 0.002)     {
        HG= 529.44008 * pow(0.002,0.108652) + 2263.5;
        ERRH2O=" HG: PRESSURE LESS THAN 0.075 MPa ";
      }
      else if (P > 21.55)     {
        HG= -22.92521* pow(21.55-18.0,2.0) +44.23671* (21.55-18.0) + 2415.01;
        ERRH2O=" HG: PRESSURE GREATER THAN 21.55 MPa ";
      }
      else if (P >= 0.002  &&  P < 0.1379)     {
        HG= 529.44008 * pow(P,0.108652) + 2263.5;
      }
      else if (P >= 0.1379 && P <= 0.348)     {
        HG= -4.0381938E-6* pow(3.0-P,15.72364) +2750.0;
      }
      else if (P > 0.348 && P <= 1.248)     {
        HG= -0.5767304* exp(-1.66153* (P-3.2)) +2800.0;
      }
      else if (P > 1.248 && P < 2.955)     {
        HG= -7.835986* (3.001-P)*(3.001-P) +2.934312*(3.001-P) +2803.71;
      }
      else if (P >= 2.955 && P <= 6.522)     {
        HG=-1.347244*(P-2.999)*(P-2.999)-2.326913*(P-2.999)+2803.35;
      }
      else if (P > 6.522 && P < 16.497)     {
        HG= -0.9219176* (P-9.0)*(P-9.0) -16.38835*(P-9.0) +2742.03;
      }
      else if (P >= 16.497 && P < 20.193)     {
        HG= -3.532177* (P-8.0)*(P-8.0) +29.81305* (P-8.0) +2565.00;
      }
      else     {
        HG= -22.92521* (P-18.0)*(P-18.0) +44.23671* (P-18.0) +2415.01;
      }
      return(HG);
}   /* end of function HG(P) */

/**********************************************************************/
float HGSUP(float P,float T)
/* This function calculates the enthalpy of light water
   in the superheated gaseous phase.*/
{
      extern char *ERRH2O;
      float TS,PP,TT,HGSUP;
      float TSAT(float P);
      float HG(float P);
      TS=TSAT(P);
      PP=P;
      TT=T;
      if (P < 0.002)     {
        PP=0.002;
        ERRH2O=" HGSUB: PRESSURE LESS THAN 0.002 MPa ";
      }
      else if (P > 18.0)     {
        PP=18.0;
        ERRH2O=" HGSUB: PRESSURE GREATER THAN 18 MPa ";
      }
      else if (T < TS)     {
        TT=TS;
        ERRH2O=" HGSUB: TEMPERATURE LESS THAN SATURATION ";
      }
      else if (T > 450.0)     {
        TT=450.0;
        ERRH2O=" HGSUB: TEMPERATURE GREATER THAN 450 deg C ";
      }
      HGSUP = HG(PP) +(4.5*PP/sqrt(7.4529E-06*pow(TT,3.0)-PP*PP)+0.28
        * exp(-0.008*(TT-162.0))-100.0/TT+2.225)*(TT-TS);
      return(HGSUP);
}   /* end of function HGSUP(P,T) */

/**********************************************************************/
float PSAT(float T)
/* This function calculates the saturation pressure of light water
   for a given temperature.*/
{
      extern char *ERRH2O;
      float PSAT;
      if (T < 17.511)     {
        PSAT=pow((17.511+99.2)/270.121,7.406365);
        ERRH2O=" PSAT: TEMPERATURE LESS THAN 17.511 deg C    ";
      }
      else if (T > 373.253)     {
        PSAT=pow((373.253+50.0)/227.2963,4.960785);
        ERRH2O=" PSAT: TEMPERATURE GREATER THAN 373.253 deg C       ";
      }
      else if (T >= 17.511  &&  T < 56.275)     {
        PSAT=pow((T+99.2)/270.121,7.406365);
      }
      else if (T >= 56.275  &&  T < 90.88)     {
        PSAT=pow((T+78.2)/254.6831,6.4058216);
      }
      else if (T >= 90.88 && T < 139.781)     {
        PSAT=pow((T+57.0)/236.2315,5.602972);
      }
      else if (T > 139.781 && T < 203.662)     {
        PSAT=pow((T+28.0)/207.9248,4.778504);
      }
      else if (T >= 203.662 && T <= 299.407)     {
        PSAT=pow((T+5.0)/185.0779,4.304376);
      }
      else if (T > 299.407 && T < 355.636)     {
        PSAT=pow((T+16.0)/195.1819,4.460843);
      }
      else       {
        PSAT=pow((T+50.0)/227.2963,4.960785);
      }
      return(PSAT);
}   /* end of function PSAT(T) */

/**********************************************************************/
float SF(float P)
/* This function calculates the specific entropy of light water
   in liquid phase at saturation for a given pressure.*/
{
      extern char *ERRH2O;
      float SF;
      if (P < 0.002)     {
        SF=4.5397665 * pow(0.002,0.0829772) - 2.449;
        ERRH2O=" SF: PRESSURE LESS THAN 0.002 MPa ";
      }
      else if (P > 21.55)     {
        SF=4.3632383E-5* pow(21.55-0.40,3.153273)+3.5;
        ERRH2O=" SF: PRESSURE GREATER THAN 21.55 MPa  ";
      }
      else if (P >= 0.002  &&  P < 0.0812)     {
        SF=4.5397665 * pow(P,0.0829772) - 2.449;
      }
      else if (P >= 0.0812 && P < 1.666)     {
        SF=3.340244*pow(P,0.125474)-1.2;
      }
      else if (P >= 1.666 && P < 8.825)     {
        SF=1.748203*pow(P,0.2275611)+0.4;
      }
      else if (P >= 8.825 && P < 16.66)     {
        SF=0.2549248*pow(P,0.6381866)+2.25;
      }
      else   {
        SF=4.3632383E-5*pow(P-0.40,3.153273)+3.5;
      }
      return(SF);
}   /* end of function SF(P) */

/**********************************************************************/
float SFSUB(float P,float T)
/* This function calculates the entropy of light water
   in the subcooled liquid phase.*/
{
      extern char *ERRH2O;
      float PS,PP,TT,SFSUB;
      float PSAT(float T);
      float SF(float P);
      PS=PSAT(T);
      PP=P;
      TT=T;
      if (P < PS)     {
        PP=PS;
        ERRH2O=" SFSUB: PRESSURE LESS THAN SATURATION ";
      }
      else if (P > 18.0)     {
        PP=18.0;
        ERRH2O=" SFSUB: PRESSURE GREATER THAN 18 MPa ";
      }
      else if (T < 17.511)     {
        TT=17.511;
        ERRH2O=" SFSUB: TEMPERATURE LESS THAN 17.511 deg C ";
      }
      else if (T > 357.03)     {
        TT=357.03;
        ERRH2O=" SFSUB: TEMPERATURE GREATER THAN 357.03 deg C ";
      }
      SFSUB = SF(PS) + (0.0004-0.325/(370.0-TT))*(PP-PS);
      return(SFSUB);
}   /* end of function SFSUB(P,T) */

/**********************************************************************/
float SG(float P)
/* This function calculates the specific entropy of light water
   in vapour phase at saturation for a given pressure.*/
{
       extern char *ERRH2O;
       float SG;
       if (P < 0.002)     {
         SG=16.644669 * pow(0.002,-0.0192733) - 10.039;
             ERRH2O=" SG: PRESSURE LESS THAN 0.002 MPa    ";
       }
       else if (P > 21.5)     {
          SG= 5.0-0.042830642*pow(21.55-18.7,1.779526);
             ERRH2O=" SG: PRESSURE GREATER THAN 21.5 MPa       ";
  }
  else if (P >= 0.002  &&  P < 0.0916)     {
          SG=16.644669 * pow(P,-0.0192733) - 10.039;
  }
      else if (P >= 0.0916 && P <= 1.48)     {
          SG=6.58681-0.335924*log(P);
  }
      else if (P > 1.48 && P <= 8.05)     {
          SG=7.8-1.227644*pow(P,0.2481072);
  }
      else if (P > 8.05 && P <= 15.64)     {
          SG=6.3-0.084638514*pow(P,0.9082161);
  }
      else if (P > 15.64 && P <= 20.0)     {
          SG=5.5-3.6897161E-3*pow(P-7.8,2.012466);
  }
      else     {
          SG= 5.0-0.042830642*pow(P-18.7,1.779526);
  }
  return(SG);
}   /* end of function SG(P) */

/**********************************************************************/
float SGSUP(float P,float T)
/* This function calculates the entropy of light water in the superheated
   gaseous phase.*/
{
  extern char *ERRH2O;
  float TS,PP,TT,SGSUP;
  TS=TSAT(P);
      PP=P;
      TT=T;
  if (P < 0.002)     {
             PP=0.002;
             ERRH2O=" SGSUP: PRESSURE LESS THAN 0.002 MPa ";
  }
  else if (P > 18.0)     {
          PP=18.0;
             ERRH2O=" SGSUP: PRESSURE GREATER THAN 18 MPa ";
  }
  else if (T < TS)     {
             TT=TS;
             ERRH2O=" SGSUP: TEMPERATURE LESS THAN SATURATION ";
  }
  else if (T > 450.0)     {
          TT=450.0;
             ERRH2O=" SGSUP: TEMPERATURE GREATER THAN 450 deg C ";
  }
  SGSUP = SG(PP)+(0.004*pow(PP,1.2)/sqrt(3.025E-11*pow(TT+46.0,5.0)-PP*PP)
         + 0.00005/sqrt(PP)-4.125E-6*TT+0.0053) * ( TT-TS );
  return(SGSUP);
}   /* end of function SGSUP(P,T) */

/**********************************************************************/
float TSAT(float P)
/* This function calculates the saturation temperature of light water for
   a given pressure.*/
{
  extern char *ERRH2O;
  float TSAT;
  if (P < 0.002)     {
          TSAT=270.121* pow(0.002,0.135019) - 99.2;
             ERRH2O=" TSAT: PRESSURE LESS THAN 0.002 MPa ";
  }
      else if (P > 21.85)     {
          TSAT= 227.2963* pow(21.85,0.201581) - 50.0;
             ERRH2O=" TSAT: PRESSURE GREATER THAN 21.85 MPa ";
  }
  else if (P >= 0.002  &&  P < 0.01672)     {
          TSAT=270.121 * pow(P,0.135019) - 99.2;
  }
  else if (P >= 0.01672  &&  P < 0.0725)     {
          TSAT=254.6831 * pow(P,0.156108) - 78.2;
  }
      else if (P >= 0.0725 && P <= 0.359)     {
          TSAT=236.2315* pow(P,0.1784767) -57.0;
  }
      else if (P > 0.359 && P <= 1.676)     {
          TSAT=207.9248* pow(P,0.2092705) -28.0;
  }
      else if (P > 1.676 && P <= 8.511)     {
          TSAT=185.0779* pow(P,0.2323217) -5.0;
  }
      else if (P > 8.511 && P <= 17.69)     {
          TSAT=195.1819* pow(P,0.2241729) - 16.0;
  }
      else       {
          TSAT=227.2963* pow(P,0.201581) - 50.0;
  }
  return(TSAT);
}   /* end of function TSAT(P) */

/**********************************************************************/
float VF(float P)
/* This function calculates the specific volume of light water
   in liquid phase at saturation for a given pressure.*/
{
  extern char *ERRH2O;
  float VF;
  if (P < 0.002)     {
          VF=1.9118E-4 * pow(0.002,0.546472) + 0.0009947;
             ERRH2O=" VF: PRESSURE LESS THAN 0.002 MPa ";
  }
      else if (P > 21.5)     {
          VF= 4.1035988E-24* pow(21.5,15.03329) +0.00189;
             ERRH2O=" VF: PRESSURE GREATER THAN 21.5 MPa ";
  }
  else if (P >= 0.002  &&  P < 0.01468)     {
          VF=1.9118E-4 * pow(P,0.546472) + 0.0009947;
  }
  else if (P >= 0.01468  &&  P < 0.275)     {
          VF=1.380934E-4 * pow(P,0.388715) + 0.000987;
  }
      else if (P >= 0.275 && P <= 1.0)     {
          VF=1.2746977E-4* pow(P,0.4644339) +0.001;
  }
      else if (P > 1.0 && P <= 3.88)     {
          VF=1.0476071E-4* pow(P,0.5651090) +0.001022;
  }
      else if (P > 3.88 && P <= 8.84)     {
             VF=3.2836717E-5* P +1.12174735E-3;
  }
      else if (P > 8.84 && P <= 14.463)     {
             VF=3.3551046E-4 * exp(0.058403566 *P) +0.00085;
  }
  else if (P > 14.463 && P < 18.052)     {
          VF= 3.1014626E-8* pow(P,3.284754) +0.00143;
  }
  else if (P >= 18.052 && P < 20.204)     {
          VF= 1.5490787E-11* pow(P,5.7205) +0.001605;
  }
      else     {
          VF= 4.1035988E-24* pow(P,15.03329) +0.00189;
  }
  return(VF);
}   /* end of function VF(P) */

/**********************************************************************/
float VFSUB(float P,float T)
/* This function calculates the specific volume of light water
   in the subcooled liquid phase.*/
{
  extern char *ERRH2O;
  float PS,PP,TT,VFSUB;
  float PSAT(float T);
  float DF(float P);
  PS=PSAT(T);
      PP=P;
      TT=T;
  if (P < PS)     {
             PP=PS;
             ERRH2O=" VFSUB: PRESSURE LESS THAN SATURATION ";
  }
  else if (P > 18.0)     {
          PP=18.0;
             ERRH2O=" VFSUB: PRESSURE GREATER THAN 18 MPa ";
  }
  else if (T < 17.511)     {
             TT=17.511;
             ERRH2O=" VFSUB: TEMPERATURE LESS THAN 17.511 deg C ";
  }
      else if (T > 357.03)     {
             TT=357.03;
             ERRH2O=" VFSUB: TEMPERATURE GREATER THAN 357.03 deg C ";
  }
  VFSUB = pow(DF(PS) + (170.0 / (375.0-TT) - 0.2) * (PP-PS ),-1.0);
  return(VFSUB);
}   /* end of function VFSUB(P,T) */

/**********************************************************************/
float VG(float P)
/* This function calculates the specific volume of light water
   in the gas phase at saturation for a given pressure.*/
{
  extern char *ERRH2O;
  float VG;
  if (P < 0.002)     {
          VG= 1.0/(5.0981616 * pow(0.002,0.936226) -0.00025);
             ERRH2O=" VG: PRESSURE LESS THAN 0.085 MPa ";
  }
      else if (P > 21.5)     {
          VG= 1.0/( 3.558734E-16* pow(21.5,13.03774) +138.0);
             ERRH2O=" VG: PRESSURE GREATER THAN 21.5 MPa ";
  }
  else if (P >= 0.002  &&  P < 0.2139)     {
          VG= 1.0/ (5.0981616 * pow(P,0.936226) -0.00025);
  }
  else if (P >= 0.2139 && P < 1.112)     {
          VG= 1.0/(5.126076* pow(P,0.9475862) +0.012);
  }
  else if (P >= 1.112 && P < 3.932)     {
          VG= 1.0/(4.630832* pow(P,1.038819) +0.52);
  }
  else if (P >= 3.932 && P < 8.996)     {
          VG= 1.0/(2.868721* pow(P,1.252148) +3.80);
  }
  else if (P >= 8.996 && P < 14.628)     {
          VG= 1.0/(0.5497653* pow(P,1.831182) +18.111);
  }
      else if (P >= 14.628 && P <= 18.21)     {
          VG=1.0/(8.5791582E-3* pow(P,3.176484) +50.0);
  }
      else if (P > 18.21 && P <= 20.253)     {
          VG= 1.0/( 3.5587113E-6* pow(P,5.660939) +88.0);
  }
      else     {
          VG= 1.0/( 3.558734E-16* pow(P,13.03774) +138.0);
  }
  return(VG);
}   /* end of function VG(P) */

/**********************************************************************/
float VGSUP(float P,float T)
/* This function calculates the specific volume of light water
   in the superheated gaseous phase.*/
{
  extern char *ERRH2O;
  float TS,PP,TT,VGSUP;
  float TSAT(float P);
  float VG(float P);
  TS=TSAT(P);
      PP=P;
      TT=T;
  if (P < 0.002)     {
             PP=0.002;
             ERRH2O=" VGSUP: PRESSURE LESS THAN 0.002 MPa ";
  }
  else if (P > 18.0)     {
          PP=18.0;
             ERRH2O=" VGSUP: PRESSURE GREATER THAN 18 MPa ";
  }
  else if (T < TS)     {
             TT=TS;
             ERRH2O=" VGSUP: TEMPERATURE LESS THAN SATURATION ";
  }
  else if (T > 450.0)     {
          TT=450.0;
             ERRH2O=" VGSUP: TEMPERATURE GREATER THAN 450 deg C ";
  }
  VGSUP = VG(PP)+(0.000466/P-(0.12/(TT+100.0) - 0.00106 )*pow(PP,0.1)
         / sqrt ( 1.96E-08 * pow(TT+8.0,4.0) -PP*PP)) * (TT-TS);
  return(VGSUP);
}   /* end of function VGSUP(P,T) */

/**********************************************************************/
float VISCF(float P)
/* dynamic viscosity - saturated liquid */
{
  float VFP,TS;
  float VF(float P);
  float TSAT(float P);
  float visco(float v,float T);
  VFP = VF(P);
  TS  = TSAT(P);
  return (visco(VFP,TS));

}   /* end of function VISCF(P,T) */

/**********************************************************************/
float VIFSUB(float P,float T)
/* dynamic viscosity - subcooled liquid */
{
  float VFP;
  float VFSUB(float P,float T);
  float visco(float v,float T);
  VFP = VFSUB(P,T);
  return(visco(VFP,T));

}   /* end of function VIFSUB(P,T) */

/**********************************************************************/
float VISCG(float P)
/* dynamic viscosity - saturated vapor */
{
  float VGP,TS;
  float VG(float P);
  float TSAT(float P);
  float visco(float v,float T);
  VGP = VG(P);
          TS  = TSAT(P);
          return (visco(VGP,TS));
}   /* end of function VISCG(P,T) */

/**********************************************************************/
float VIGSUP(float P,float T)
/* dynamic viscosity - superheated vapor  */
{
          float VGP;
          float VGSUP(float P,float T);
          float visco(float v,float T);
          VGP = VGSUP(P,T);
          return(visco(VGP,T));
}   /* end of function VIGSUP(P,T) */

/**********************************************************************/
float visco(float vol,float T)
/* dynamic viscosity of water and stream - IAPS '75 formulation
                                           for scientific purposes
                        input : vol   - specific volume (m^3/kg)
                                 T     - tmperature (0C)
                        output: visco - dynamic viscosity (Pa*s*10^-6) */
{
          float tred,rhored,xmi,tb,rob,a,b,c,d,e,xm,rm,r0,tk,dmi,visco;
          tred = (T + 273.15) / 647.27;
          rhored = 1.0/ vol / 317.763;
          xmi = 100.0*sqrt(tred)/(((-0.36744/tred+1.05287)/tred+1.77624)/tred+1.81583);
          tb = 1.0/tred-1.0;
          rob = rhored-1.0;
          a = ((((0.146543 * tb-0.551119) * tb+0.907919) * tb-0.130356) * tb+0.162888) * tb+0.501938;
          b = ((((-0.084337 * tb+0.0670665) * tb+1.207552) * tb+0.6736651) * tb+0.789393)*tb+0.235622;
          c = ((((0.195286 * tb-0.497089) * tb-0.687343) * tb-0.959456) * tb-0.743539) * tb-0.274637;
          d = ((((-0.032932 * tb+0.100754) * tb+0.213486) * tb+0.347247) * tb+0.263129) * tb+0.145831;
          e = ((((-0.0202595 * tb+0.0602253) * tb-0.0822904) *tb-0.0267758)* tb-0.0253093) * tb-0.0270448;
      xm = (((e*rob+d)*rob+c)*rob+b)*rob+a;
      xm = xmi*exp(rhored*xm);
          if (T >= 367.0  &&  T <= 380.0) {
                 rm = 317.763+107.661*pow(ABS(tred-1.0),0.067174)*exp(-543.48*ABS(tred-1.0));
         r0 = sqrt(3.88937+5.11063/tred);
                 tk = 1.0/ABS(vol*rm*tred*pow(r0/(r0-1.0/vol/rm),2)-2.25);
                 dmi = 0.01204*xmi*tk*exp(-102.4*pow(rob,4.0)-5279.*pow(tred-1.0,2.0));
                 dmi = dmi+0.1432*xmi*exp(-1107.*pow(rob,4.0)-3.08e5*pow(tred-1.0,2.0));
         visco = xm + dmi;
          }
          else                          {
         visco = xm;
          }
          return(visco);
}   /* end of function visco(VOL,T) */

/**********************************************************************/
float TCF(float P)
/* thermal conductivity - saturated liquid */
{
          float VFP,TS;
          float VF(float P);
          float TSAT(float P);
          float tcond(float v,float T);
          VFP = VF(P);
          TS  = TSAT(P);
          return(tcond(VFP,TS));
}   /* end of function TCF(P) */

/**********************************************************************/
float TCFSUB(float P,float T)
/* thermal conductivity - subcooled liquid */
{
          float VFP;
          float VFSUB(float P,float T);
          float tcond(float v,float T);
          VFP = VFSUB(P,T);
          return (tcond(VFP,T));
}   /* end of function TCFSUB(P,T) */

/**********************************************************************/
float TCG(float P)
/* thermal conductivity - saturated vapor */
{
          float VGP,TS;
          float VG(float P);
          float TSAT(float P);
          float tcond(float v,float T);
          VGP = VG(P);
          TS  = TSAT(P);
          return (tcond(VGP,TS));

}   /* end of function TCG(P) */

/**********************************************************************/
float TCGSUP(float P,float T)
/* thermal conductivity - superheated vapor */
{
          float VGP;
          float VGSUP(float P,float T);
          float tcond(float v,float T);
          VGP = VGSUP(P,T);
          return (tcond(VGP,T));
}   /* end of function TCGSUP(P,T) */

/**********************************************************************/
float tcond(float vol,float T)
/* thermal conductivity of water and steam - IAPS '77 formulation
                                             for industrial purposes
                        input : vol   - specific volume (m^3/kg)
                             T     - temperature (0C)
                        output: TCOND - thermal conductivity (W/m*K) */
{
          float tred,rhored,xla0,xlas,dt,q,s,r,d1,d2,d3,TCOND;
          tred = (T + 273.15) / 647.3;
          rhored = 1.0/ vol / 317.7;
          xla0 = (((-4.22464e-3*tred+1.56146e-2)*tred+2.99621e-2)*tred+1.02811e-2)*sqrt(tred);
          xlas = -0.39707+0.400302*rhored+1.06*exp(-0.171587*pow(rhored+2.39219,2.0));
          dt = ABS(tred-1.0)+3.08976e-3;
          q = pow(dt,-0.6);
          if (tred >= 1.0) s = 1.0/dt;
          if (tred<1.0) s = 10.0932*q;
          q = 2.0+8.22994e-2*q;
          r = q+1.0;
          d1 = (7.01309e-2*pow(tred,-10.0)+1.1852e-2)*pow(rhored,1.8)*exp(0.642857*(1.0-pow(rhored,2.8)));
          d2 = 1.69937e-3*s*pow(rhored,q)*exp(q*(1.0-pow(rhored,r))/r);
          d3 = 4.11717*tred*sqrt(tred)+6.17937*pow(rhored,-5.0);
          if ( d3 > 88.7228 )   {    /* check for maximum onent */
                  d3 = 0.0;
          }
          else                  {
                  d3 = -1.02 * exp(-d3);
          }
          TCOND = xla0+xlas+d1+d2+d3;
          return(TCOND);
}   /* end of function tcond(VOL,T) */

/**********************************************************************/
float SIGMAF(float T)
/* surface tension of water - IAPS '76, Release on surface tension of
                              water substance. IAPS, Washington: NBS */
{
      float Tabs,Tcrit,Theta;
          Tabs=T+273.15;
          Tcrit=647.15;
          Theta=(Tcrit-Tabs)/Tcrit;
          return(0.2356*(1.0 - 0.625*Theta)*pow(Theta,1.256));
} /* of function SIGMAF */
